package simulation;

import simulation.avl.*;


public class Simulation {
    TeltonikaParameters parameters;
    SimulationClient client;

    private Avl avl;

    private final String imei;
    private final long interval;
    private long avlTime;
    private long packetTime;

    Simulation(String ip, String imei, float interval) {
        parameters = new TeltonikaParameters("./res/FMC130_parameters.csv");
        client = new SimulationClient(ip, 8160);

        this.avl = new Avl();
        this.imei = imei;
        this.interval = (long) (interval * 60000);

        avlTime = System.currentTimeMillis();
        packetTime = 0;
    }

    void start() {
        if (!client.connect(getImei())) System.exit(0);
        client.start();
    }

    void loop() {
        int packetCount = 2;
        while (client.isConnected()) {
            if (System.currentTimeMillis() - packetTime > interval / packetCount) {
                avl.addAvlPackage(generateAvlPackage());
                packetTime = System.currentTimeMillis();
            }

            if (System.currentTimeMillis() - avlTime > interval) {
                System.out.println("Sending avl package...");
                client.sendMessage(avl.toString());
                System.out.println("Sent");
                avl = new Avl();
                avlTime = System.currentTimeMillis();
            }
        }
    }

    public String getImei() {
        return Avl.intToHex(imei.length(), 2) + Avl.stringToHex(imei);
    }

    public Gps generateGps() {
        long longitude = (long) ((Math.random() * 900000000));
        long latitude = (long) ((Math.random() * 900000000));
        int altitude = (int) (Math.random() * 1000);
        int angle = (int) (Math.random() * 359);
        int satellites = (int) (Math.random() * 12);
        int speed = (int) (Math.random() * 100);

        return new Gps(
                longitude,
                latitude,
                altitude,
                angle,
                satellites,
                speed);
    }

    public IoElements generateIoElements() {
        IoElements ioElements = new IoElements();

        int count = (int) (20 + Math.random() * 30);
        for (int i = 0; i < count; i++) ioElements.add(generateIo());

        return ioElements;
    }

    public Io generateIo() {
        int id;
        do {
            if (Math.random() < 0.5) id = (int) (Math.random() * 858);
            else id = (int) (161 + Math.random() * 11);
        } while (!parameters.idExists(id));

        TeltonikaParameters.IoElementParameter parameter = parameters.getElement(id);

        long value = 1;
        int bytes = parameter.getBytes();

        boolean special = bytes != 1 && bytes != 2 && bytes != 4 && bytes != 8;

        if (bytes == 0) bytes = (int) (Math.random() * 10);

        try {
            long min = Long.parseLong(parameter.getMin());
            long max = Long.parseLong(parameter.getMax());
            value = (long) (min + Math.random() * (max - min));

        } catch (NumberFormatException ignored) {

        }

        return new Io(id, value, bytes, special);
    }

    public AvlPackage generateAvlPackage() {
        AvlPackage avlPackage = new AvlPackage(System.currentTimeMillis(), 0);
        avlPackage.setGps(generateGps());
        avlPackage.setIoElements(generateIoElements());
        return avlPackage;
    }

    public static void main(String[] args) {
        String ip = "localhost";
        String imei = "01";
        float interval = 0.001f;

        if (args.length < 3) {
            System.out.println("Missing system arguments. Using default values: IP-address: " + ip + " IMEI: " + imei + " Update interval: " + interval);
        } else {
            ip = args[0].toLowerCase();
            imei = args[1].toLowerCase();
            try {
                interval = Float.parseFloat(args[2]);
            } catch (NumberFormatException e) {
                System.out.println("IMEI/Update interval is not a number");
                return;
            }
        }
        Simulation simulation = new Simulation(ip, imei, interval);
        simulation.start();
        simulation.loop();
    }
}

