package simulation;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.*;

/**
 * Denne klassen inneholder parameter-id med info til disse teltonikasensorene:
 * FMB0XY, FMB1XY, FMB9XY, FMB96X, FM3001, FMT100, TMT250,
 * FMC1XY, FMC0XY, FMUXY, FMM1XY, FMP100
 */
public class TeltonikaParameters {
    private final HashMap<Integer, IoElementParameter> parameters;

    public TeltonikaParameters(String url) {
        parameters = new HashMap<>();
        if (!readCsv(url)) System.out.println("\nKlarte ikke lese CSV fil");
    }

    /**
     * Klasse for en IOparameter
     */
    public static class IoElementParameter {
        private final int id;
        private final String name;
        private final int bytes;
        private final String type;
        private final String min;
        private final String max;
        private final double multiplier;
        private final double limit;

        // Konstruktør for å lage et element
        IoElementParameter(int id, String name, int bytes, String type, String min, String max, double multiplier, double limit) {
            this.id = id;
            this.name = name;
            this.bytes = bytes;
            this.type = type;
            this.min = min;
            this.max = max;
            this.multiplier = multiplier;
            this.limit = limit;
        }

        // Hente info
        public int getId() {
            return id;
        }

        public String getName() {
            return name;
        }

        public int getBytes() {
            return bytes;
        }

        public String getType() {
            return type;
        }

        public String getMin() {
            return min;
        }

        public String getMax() {
            return max;
        }

        public double getMultiplier() {
            return multiplier;
        }

        public double getLimit() {
            return limit;
        }

        @Override
        public String toString() {
            return "\nParameter med id " + id +
                    "\n\tid: " + id
                    + "\n\tname: " + name
                    + "\n\tbytes: " + bytes
                    + "\n\ttype: " + type
                    + "\n\tmin: " + min
                    + "\n\tmax: " + max
                    + "\n\tmultiplier: " + multiplier
                    + "\n";
        }
    }

    /**
     * Hent IoElement i fra id
     */
    public IoElementParameter getElement(int id) throws IllegalArgumentException {
        if (!parameters.containsKey(id))
            throw new IllegalArgumentException("IoElement med id " + id + ", finnes ikke.");
        return parameters.get(id);
    }

    public boolean idExists(int id) {
        return parameters.containsKey(id);
    }

    /**
     * Leser CSV-fil og fyller inn i HashMap
     */
    public boolean readCsv(String url) {
        Scanner s = null;
        try {
            s = new Scanner(new File(url));
        } catch (FileNotFoundException e) {
            System.out.println("Fant ikke filen med addresse: " + url);
        }
        if (s == null) return false;

        // Hopp over første linja (som har parameter-navn)
        if (s.hasNextLine()) s.nextLine();

        // Linjenummer
        int ln = 2;

        while (s.hasNextLine()) {
            List<String> values = Arrays.asList(s.nextLine().split(","));
            if (values.size() != 8) {
                printFileError(url, "For " + (values.size() > 8 ? "mange" : "få") + " argumenter på linje " + ln);
                return false;
            }

            int id, bytes;
            String name, type, min, max;
            double multiplier, limit;

            try {
                id = Integer.parseInt(values.get(0));
            } catch (NumberFormatException e) {
                printFileError(url, "Feil format for 'id' med type 'Integer' på linje " + ln);
                return false;
            }

            name = values.get(1);

            try {
                bytes = (!values.get(2).equals("NaN")) ? Integer.parseInt(values.get(2)) : 0;
            } catch (NumberFormatException e) {
                printFileError(url, "Feil format for 'bytes' med type 'Integer' på linje " + ln);
                return false;
            }

            type = values.get(3);
            min = values.get(4);
            max = values.get(5);

            try {
                multiplier = Double.parseDouble(values.get(6));
            } catch (NumberFormatException e) {
                printFileError(url, "Feil format for 'multiplier' med type 'Double' på linje " + ln);
                return false;
            }

            try {
                limit = Double.parseDouble(values.get(7));
            } catch (NumberFormatException e) {
                printFileError(url, "Feil format for 'multiplier' med type 'Double' på linje " + ln);
                return false;
            }
            parameters.put(id, new IoElementParameter(id, name, bytes, type, min, max, multiplier, limit));

            ln++;
        }
        s.close();
        return true;
    }

    void printFileError(String url, String e) {
        System.out.println("Feil i fil '" + url + "':\n\t" + e + "\n");
    }
}
