package database;

import dataprocessing.output.PacketContainer;
import dataprocessing.output.Output;
import dataprocessing.output.RawData;
import dataprocessing.output.equipment.Equipment;
import tools.Logger;

import java.io.FileInputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import java.util.Properties;

/**
 * Grensesnittet til databasen.
 * Har funksjoner for lagring av prosessertdata og validering av id
 */
public class Database {
    private static Database instance;

    private String url;
    private String username;
    private String password;

    private Database() {

    }

    /**
     * Stiastisk funksjon som henter singleton intanse av Databasen.
     * Følger singleton lazy initialization.
     *
     * @return Database
     */
    public static Database getInstance() {
        if (instance == null)  instance = new Database();
        return instance;
    }

    /**
     * Initierer databasen. Tar inn en Ressurs-fil som inneholder urlen, brukernavnet og passord til databasen
     *
     * @param filename Lokasjonsaddresse til Ressurs-fil
     */
    public void init(String filename){
        var props = new Properties();

        try (var in = new FileInputStream(filename)) {
            props.load(in);

            url = props.getProperty("jdbc.url");
            username = props.getProperty("jdbc.username");
            password = props.getProperty("jdbc.password");

            if (username == null) username = "";
            if (password == null) password = "";

        } catch (IOException e) {
            Logger.writeError("Finner ikke: " + filename);
        }
    }

    /**
     * Returnerer et tilkoblingsobjekt basert på informasjonen lest fra Ressurs-filen
     *
     * @return Tilkoblingsobjektet som skal brukes til SQL-spørringer
     * @throws SQLException
     */
    public Connection getConnection() throws SQLException {
        return DriverManager.getConnection(url, username, password);
    }

    /**
     * Lagrer prosessert data til datbasen
     *
     * @param data Den prosessert dataen
     * @return Hvis largingen er vellykket returners parameteren ellers returneres NULL
     */
    public Output processData(Output data) {
        if (data == null) return null;
        try {
            var connection = getConnection();
            for (PacketContainer packet : data.getDataPackets()) {
                int id = ParsedDataSqlStatements.addDataPacket(connection, packet, data.getDeviceId());
                if (id >= 0) {
                    try {
                        ParsedDataSqlStatements.addGpsElement(connection, packet.getGpsElement(), id);
                        for (Equipment equipment : packet.getEquipmentArray()) {
                            int eId = ParsedDataSqlStatements.addEquipment(connection, equipment, id);
                            for (RawData rawData : equipment.getRawData()) {
                                ParsedDataSqlStatements.addRawData(connection, rawData, eId);
                            }
                        }
                    } catch (SQLException e) {
                        Logger.writeWarning("Feilet å logge data");
                        try{
                            ParsedDataSqlStatements.removeDataPacket(connection, id);
                            Logger.writeWarning("Sletter data med id: " + id);
                        }catch (SQLException ignore) {
                            Logger.writeWarning("Feilet å slette data med id: " + id);
                        }
                        return null;
                    }
                }
            }
            connection.close();
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        }
        return data;
    }

    /**
     * Sjekker om IDen finnes i databsen
     *
     * @param id IDen som skal valideres
     * @return ID-en eller -1 hvis der er en ugyldig ID
     */
    public long validateId(long id) {
        try {
            return ParsedDataSqlStatements.idExists(getConnection(), id);
        } catch (SQLException e) {
            Logger.writeToConsole("Failed to connect to database");
            e.printStackTrace();
        }
        return -1;
    }
}



