package database;

import dataprocessing.output.PacketContainer;
import dataprocessing.output.equipment.Equipment;
import dataprocessing.output.GpsElement;
import dataprocessing.output.RawData;

import java.sql.*;

/**
 * Klasse med statiske prepared statements
 */
public class ParsedDataSqlStatements {

    private ParsedDataSqlStatements() {}

    /**
     * SQl-spørring for å legge til datapakker
     *
     * @param connection Kommunikasjonselement til database
     * @param packet     DataPakke Element
     * @param sensorId   ID til sensor
     * @return ID-til datapakken i databasen, hvis den feiler returneres -1
     * @throws SQLException
     */
    public static int addDataPacket(Connection connection, PacketContainer packet, long sensorId) throws SQLException {
        try (PreparedStatement ps = connection.prepareStatement(
                "INSERT INTO datapacket (`device_id`, `timestamp`) VALUES (?,?)",
                Statement.RETURN_GENERATED_KEYS)) {

            ps.setLong(1, sensorId);
            ps.setTimestamp(2, new Timestamp(packet.getDateAndTime()));

            int rows = ps.executeUpdate();
            if (rows == 0) return -1;

            ResultSet r = ps.getGeneratedKeys();
            r.next();

            return r.getInt(1);
        }
    }

    /**
     * SQl-spørring for å fjerne datapakke fra databasen
     *
     * @param connection Kommunikasjonselement til database
     * @param id         ID-en til elementet som skal fjernes
     * @throws SQLException
     */
    public static void removeDataPacket(Connection connection, int id) throws SQLException {
        try (PreparedStatement ps = connection.prepareStatement("DELETE FROM `datapacket` WHERE id=?")) {
            ps.setInt(1, id);
            ps.executeUpdate();
        }
    }

    /**
     * SQl-spørring for å legge til IO-event(Equipment) til databasen
     *
     * @param connection   Kommunikasjonselement til database
     * @param equipment    Equipment id
     * @param dataPacketID ID til datapakken
     * @return ID-til IOelementet i databasen
     * @throws SQLException
     */
    public static int addEquipment(Connection connection, Equipment equipment, int dataPacketID) throws SQLException {
        try (PreparedStatement ps = connection.prepareStatement(
                "INSERT INTO io_event (`datapacket_id`, `event_name`, `value`) VALUES (?,?,?)",
                Statement.RETURN_GENERATED_KEYS)) {
            ps.setInt(1, dataPacketID);
            ps.setString(2, equipment.getName());
            ps.setString(3, equipment.getState());
            ps.executeUpdate();


            ResultSet r = ps.getGeneratedKeys();
            r.next();
            return r.getInt(1);
        }

    }

    /**
     * SQl-spørring for å legge til rådata knyttet sammen med IO-events (Equipment)
     *
     * @param connection  Kommunikasjonselement til database
     * @param raw         Den råe dataen
     * @param equipmentId ID-en til IOelementet (Equipment)
     * @throws SQLException
     */
    public static void addRawData(Connection connection, RawData raw, int equipmentId) throws SQLException {
        try (PreparedStatement ps = connection.prepareStatement("INSERT INTO raw_data (`io_event_id`, `name`, `value`) VALUES (?,?,?)")) {
            ps.setInt(1, equipmentId);
            ps.setString(2, raw.getName());
            ps.setString(3, raw.getValue());
            ps.executeUpdate();
        }
    }

    /**
     * SQl-spørring for å legge til GPS-element i Database
     *
     * @param connection   Kommunikasjonselement til database
     * @param gps          GPS-elementet
     * @param dataPacketID ID til datapakken
     * @throws SQLException
     */
    public static void addGpsElement(Connection connection, GpsElement gps, int dataPacketID) throws SQLException {
        try (PreparedStatement ps = connection.prepareStatement("INSERT INTO gps_data (`datapacket_id`, `longitude`, `latitude`, `altitude`, `angle`, `no_of_satellites`) VALUES (?,?,?,?,?,?)")) {
            ps.setInt(1, dataPacketID);
            ps.setDouble(2, gps.getLongitude());
            ps.setDouble(3, gps.getLatitude());
            ps.setShort(4, gps.getAltitude());
            ps.setShort(5, gps.getAngle());
            ps.setByte(6, gps.getNoOfSatellites());
            ps.executeUpdate();
        }
    }

    /**
     * SQl-spørring for å sjekke om iden finnes i databasen
     *
     * @param connection Kommunikasjonselement til database
     * @param id         iden som skal sjekkes
     * @return Retrunerer Id hvis den finnes eller -1
     * @throws SQLException
     */
    public static long idExists(Connection connection, long id) throws SQLException {
        try (PreparedStatement ps = connection.prepareStatement("SELECT COUNT(id) FROM device where id=?")) {
            ps.setLong(1, id);
            ResultSet result = ps.executeQuery();
            result.next();
            if (result.getBoolean(1)) return id;
            return -1;
        }
    }
}
