package dataprocessing;

import dataprocessing.output.Output;
import dataprocessing.teltonikafmc.TeltonikaFmcProtocol;
import networking.Packet;
import tools.DeviceFamily;

import java.util.EnumMap;

/**
 * Grensesnitt til Dataprocesseringmodulen.
 * Denne klassen skal velge hvilken protocoll som skal kjøres utifra Packet-type
 */
public class ProtocolHandler {
    private static ProtocolHandler instance;

    private final EnumMap<DeviceFamily, Protocol> protocols = new EnumMap<>(DeviceFamily.class);

    private ProtocolHandler() {}

    /**
     * Stiastisk funksjon som henter singleton intanse av ProtocolHandler.
     * Følger singleton lazy initialization.
     *
     * @return ProtocolHandler
     */
    public static ProtocolHandler getInstance() {
        if (instance == null) instance = new ProtocolHandler();
        return instance;
    }

    /**
     * Legegr til protokoller i listen
     */
    public void init(){
        protocols.put(DeviceFamily.TELTONIKA_FMC, new TeltonikaFmcProtocol());
    }

    /**
     * Henter protokollklassen ut ifra typen til enhet
     *
     * @param id sensor type
     * @return protokollen
     */
    private Protocol getProtocol(DeviceFamily id) {
        return protocols.get(id);
    }

    /**
     * Grensesnitt til validate funksjone til protokollen
     *
     * @param packet pakken som skal klassifiseres
     * @return pakkens klassifikasjon
     */
    public Protocol.Codes validateData(Packet packet) {
        return getProtocol(packet.getDeviceFamily()).validateData(packet);
    }

    /**
     * Grensesnitt til handshake funksjonen til protokollen
     *
     * @param packet handshakepakke
     * @return IDen til enheten eller -1
     */
    public long processHandshake(Packet packet) {
        return getProtocol(packet.getDeviceFamily()).processHandshake(packet);
    }

    /**
     * Grensesnitt til prosesserdata funksjonen til protokollen
     *
     * @param packet Datapakke
     * @return Output objekt med tolket data eller null
     */
    public Output processData(Packet packet) {
        return getProtocol(packet.getDeviceFamily()).processData(packet);
    }
}