package dataprocessing.teltonikafmc;

import dataprocessing.output.PacketContainer;
import dataprocessing.output.equipment.Plow;
import dataprocessing.output.equipment.SaltSpreader;
import dataprocessing.output.equipment.SaltTank;
import tools.ByteStreamReader;

import java.util.HashMap;

/**
 * Klasse som holder funksjoner for a dekode io data fra TeltonikaFMC enheter.
 */
public class TeltonikaFmcDecoder {
    private static final int DEFAULT = 0;

    private final HashMap<Integer, Decoder> decoders = new HashMap<>();

    /**
     * Interface for dekoder funksjoner
     */
    private interface Decoder {
        void decode(ByteStreamReader reader, PacketContainer packet, TeltonikaFmcParameters.IoElementParameter parameters) throws ReflectiveOperationException;
    }

    TeltonikaFmcDecoder() {
        /*
         * Standard dekoder funksjon.
         * Hopper over bytes i strømen
         */
        decoders.put(DEFAULT, (reader, packet, parameters) -> {
            int bytes = parameters.getBytes();
            if (bytes != 1 && bytes != 2 && bytes != 4 && bytes != 8) bytes = reader.readInt16();
            reader.skip(bytes);
        });

        /*
         * Dekoder for lesing av IO hendelse med id 161 (Slope of Arm)
         */
        decoders.put(161, (reader, packet, parameters) -> {
            var value = reader.readInt16() * parameters.getMultiplier();

            var plow = packet.getEquipment(Plow.class);
            plow.addRawData(parameters.getName(), String.valueOf(value));

            plow.setDown(value < parameters.getLimit());
        });

        /*
         * Dekoder for lesing av IO hendelse med id 162 (Rotation of Arm)
         */
        decoders.put(162, (reader, packet, parameters) -> {
            var value = reader.readInt16() * parameters.getMultiplier();

            var plow = packet.getEquipment(Plow.class);
            plow.addRawData(parameters.getName(), String.valueOf(value));

            plow.setDown(value < parameters.getLimit());
        });

        /*
         * Dekoder for lesing av IO hendelse med id 163 (Eject of Arm)
         */
        decoders.put(163, (reader, packet, parameters) -> {
            var value = reader.readInt16() * parameters.getMultiplier();

            var plow = packet.getEquipment(Plow.class);
            plow.addRawData(parameters.getName(), String.valueOf(value));

            plow.setDown(value < parameters.getLimit());
        });

        /*
         * Dekoder for lesing av IO hendelse med id 164 (Horizontal Distance Arm)
         */
        decoders.put(164, (reader, packet, parameters) -> {
            var value = reader.readInt16() * parameters.getMultiplier();

            var plow = packet.getEquipment(Plow.class);
            plow.addRawData(parameters.getName(), String.valueOf(value));

            plow.setDown(value < parameters.getLimit());
        });

        /*
         * Dekoder for lesing av IO hendelse med id 165 (Height Arm Above Ground)
         */
        decoders.put(165, (reader, packet, parameters) -> {
            var value = reader.readInt16() * parameters.getMultiplier();

            var plow = packet.getEquipment(Plow.class);
            plow.addRawData(parameters.getName(), String.valueOf(value));

            plow.setDown(value < parameters.getLimit());
        });

        /*
         * Dekoder for lesing av IO hendelse med id 167 (Spread Salt)
         */
        decoders.put(167, (reader, packet, parameters) -> {
            var value = reader.readInt16() * parameters.getMultiplier();

            var saltSpreader = packet.getEquipment(SaltSpreader.class);
            saltSpreader.addRawData(parameters.getName(), String.valueOf(value));

            saltSpreader.setOn(value > parameters.getLimit());
        });

        /*
         * Dekoder for lesing av IO hendelse med id 169 (Spread Fine Grained Salt)
         */
        decoders.put(169, (reader, packet, parameters) -> {
            var value = reader.readInt32() * parameters.getMultiplier();

            var saltSpreader = packet.getEquipment(SaltSpreader.class);
            saltSpreader.addRawData(parameters.getName(), String.valueOf(value));

            saltSpreader.setOn(value > parameters.getLimit());
        });

        /*
         * Dekoder for lesing av IO hendelse med id 170 (Coarse Grained Salt)
         */
        decoders.put(170, (reader, packet, parameters) -> {
            var value = reader.readInt32() * parameters.getMultiplier();

            var saltSpreader = packet.getEquipment(SaltSpreader.class);
            saltSpreader.addRawData(parameters.getName(), String.valueOf(value));

            var saltTank = packet.getEquipment(SaltTank.class);
            saltTank.addRawData(parameters.getName(), String.valueOf(value));

            saltSpreader.setOn(value > parameters.getLimit());
            saltTank.setLevel((int) value);
        });
    }

    /**
     * Finner hvilken dekoder som skal brukes for gitt IOevent og tolker dataen i PacketConatineren.
     * Er det ingen dekoder brukes default dekoderen.
     *
     * @param id         Id til IO evntet
     * @param reader     bytestreamen fra sensoren
     * @param packet     PacketContainere som dataen skal lagres i
     * @param parameters Io parameterne
     * @throws  ReflectiveOperationException kaster en exception hvis det er noe galt med henting av utstyrsklasse
     */
    public void decode(int id, ByteStreamReader reader, PacketContainer packet, TeltonikaFmcParameters.IoElementParameter parameters) throws ReflectiveOperationException {
        if (decoders.containsKey(id)) decoders.get(id).decode(reader, packet, parameters);
        else decoders.get(DEFAULT).decode(reader, packet, parameters);
    }
}
