package dataprocessing.teltonikafmc;

import tools.Logger;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.*;

/**
 * Denne klassen inneholder parameter-id med info til disse teltonikaenhetene:
 * FMB0XY, FMB1XY, FMB9XY, FMB96X, FM3001, FMT100, TMT250,
 * FMC1XY, FMC0XY, FMUXY, FMM1XY, FMP100
 */
public class TeltonikaFmcParameters {
    private final HashMap<Integer, IoElementParameter> parameters;

    public TeltonikaFmcParameters(String url) {
        parameters = new HashMap<>();
        if (!readCsv(url)) Logger.writeToConsole("\nKlarte ikke lese CSV fil");
    }

    /**
     * Klasse for en IO parameter
     */
    public static class IoElementParameter {
        private final int id;
        private final String name;
        private final int bytes;
        private final String type;
        private final String min;
        private final String max;
        private final double multiplier;
        private final double limit;

        IoElementParameter(int id, String name, int bytes, String type, String min, String max, double multiplier, double limit) {
            this.id = id;
            this.name = name;
            this.bytes = bytes;
            this.type = type;
            this.min = min;
            this.max = max;
            this.multiplier = multiplier;
            this.limit = limit;
        }

        /**
         * @return IO hendelsens iden
         */
        public int getId() {
            return id;
        }

        /**
         * @return navnet på IO hendelsen
         */
        public String getName() {
            return name;
        }

        /**
         * @return antall Bytes io hendelsen tar i byte strømen
         */
        public int getBytes() {
            return bytes;
        }

        /**
         * @return hvilken type dataen er (Int, Unsigned, Byte,...)
         */
        public String getType() {
            return type;
        }

        /**
         * @return minste veriden IO hendelsen kan være
         */
        public String getMin() {
            return min;
        }

        /**
         * @return største veriden IO hendelsen kan være
         */
        public String getMax() {
            return max;
        }

        /**
         * @return multiplikator verdien skal multipleres med
         */
        public double getMultiplier() {
            return multiplier;
        }

        /**
         * Grensen mellom stadier.
         * F.eks. Plogen er nede hvis verdien er mindre en grensen eller opp hvis den er større.
         *
         * @return grensen
         */
        public double getLimit() {
            return limit;
        }

        @Override
        public String toString() {
            return "\nParameter med id " + id +
                    "\n\tid: " + id
                    + "\n\tname: " + name
                    + "\n\tbytes: " + bytes
                    + "\n\ttype: " + type
                    + "\n\tmin: " + min
                    + "\n\tmax: " + max
                    + "\n\tmultiplier: " + multiplier
                    + "\n";
        }
    }


    /**
     * Henter IO parameter
     * @param id iden til IO hendelsen
     * @return IO parameter
     * @throws IllegalArgumentException
     */
    public IoElementParameter getElement(int id) throws IllegalArgumentException {
        if (!parameters.containsKey(id))
            throw new IllegalArgumentException("IoElement med id " + id + ", finnes ikke.");
        return parameters.get(id);
    }


    /**
     * Leser CSV-fil og fyller inn i HashMap
     *
     * @param url Lokasjonsaddresse til csv-filen
     * @return True hvis den klarer å lese filen
     */
    public boolean readCsv(String url) {
        Scanner s;
        try {
            var file = new File(url);
            s = new Scanner(file);
        } catch (FileNotFoundException e) {
            Logger.writeToConsole("Fant ikke filen med addresse: " + url);
            return false;
        }

        // Hopp over første linja (som har parameter-navn)
        if (s.hasNextLine()) s.nextLine();

        // Linjenummer
        var ln = 2;

        while (s.hasNextLine()) {
            List<String> values = Arrays.asList(s.nextLine().split(","));
            if (values.size() != 8) {
                printFileError(url, "For " + (values.size() > 8 ? "mange" : "få") + " argumenter på linje " + ln);
                return false;
            }

            int id;
            int bytes;
            String name;
            String type;
            String min;
            String max;
            double multiplier;
            double limit;

            try {
                id = Integer.parseInt(values.get(0));
            } catch (NumberFormatException e) {
                printFileError(url, "Feil format for 'id' med type 'Integer' på linje " + ln);
                return false;
            }

            name = values.get(1);

            try {
                bytes = (!values.get(2).equals("NaN")) ? Integer.parseInt(values.get(2)) : 0;
            } catch (NumberFormatException e) {
                printFileError(url, "Feil format for 'bytes' med type 'Integer' på linje " + ln);
                return false;
            }

            type = values.get(3);
            min = values.get(4);
            max = values.get(5);

            try {
                multiplier = Double.parseDouble(values.get(6));
            } catch (NumberFormatException e) {
                printFileError(url, "Feil format for 'multiplier' med type 'Double' på linje " + ln);
                return false;
            }

            try {
                limit = Double.parseDouble(values.get(7));
            } catch (NumberFormatException e) {
                printFileError(url, "Feil format for 'multiplier' med type 'Double' på linje " + ln);
                return false;
            }
            parameters.put(id, new IoElementParameter(id, name, bytes, type, min, max, multiplier, limit));

            ln++;
        }
        s.close();
        return true;
    }

    /**
     * Skriver ut hvis det er noe galt ved lesing av fil
     * @param url Lokasjonsaddresse til csv-filen
     * @param str meldingen
     */
    void printFileError(String url, String str) {
        Logger.writeToConsole("Feil i fil '" + url + "':\n\t" + str + "\n");
    }
}
