package dataprocessing.teltonikafmc;

import dataprocessing.output.PacketContainer;
import dataprocessing.Protocol;
import tools.ByteStreamReader;
import tools.Crc;
import tools.DataConverter;
import networking.Packet;
import dataprocessing.output.Output;
import tools.Logger;

import java.util.Arrays;

/**
 * Implementasjon av protokollklassen for TeltonikaFMC.
 * Implementert Codec8 og Codec8E
 */
public class TeltonikaFmcProtocol implements Protocol {
    private final TeltonikaFmcParameters parameters;
    private final TeltonikaFmcDecoder decoder;

    public TeltonikaFmcProtocol() {
        parameters = new TeltonikaFmcParameters("./FMC130_parameters.csv");
        decoder = new TeltonikaFmcDecoder();
    }

    /**
     * Sjekker om pakken er en gyldig TeltonikaFMC pakke og klassifiserer pakken
     *
     * @param packet TeltonikaFMC packet
     * @return Pakkens klassifikasjon
     */
    @Override
    public Codes validateData(Packet packet) {
        byte[] data;
        try {
            data = packet.getData();
        } catch (ClassCastException e) {
            return Codes.ERROR;
        }

        int len = data.length;

        if (len < 4) {
            Logger.writeWarning("Pakke fra client er mindre enn 4 bytes");
            return Codes.ERROR;
        }

        int expectedLen = DataConverter.byteToInt16(data, 0);

        if (data[0] != 0 || data[1] != 0 || data[2] != 0 || data[3] != 0) {
            if (len != (expectedLen + 2)) return Codes.ERROR;
            return Codes.HANDSHAKE;
        }

        if (len < 8) {
            Logger.writeWarning("Pakke fra client er mindre enn 8 bytes");
            return Codes.ERROR;
        }

        var avlLen = DataConverter.byteToInt32(data, 4);

        if (len != avlLen + 12) {
            Logger.writeWarning("Pakke fra client er mindre/større enn avl + 12");
            return Codes.ERROR;
        }

        if (Crc.crcChecker(data) != DataConverter.byteToInt32(data, (len - 4))) {
            Logger.writeWarning("CRC er feil: " + Crc.crcChecker(data) + " " + DataConverter.byteToInt32(data, (len - 4)));
            return Codes.ERROR;
        }

        return Codes.DATA;
    }

    /**
     * Sjekker om pakken følger handshake formatet og leser IMEI-nummeret til enheten
     *
     * @param packet TeltonikaFMC packet
     * @return IMEI-nummeret til enheten eller -1 hvis noe var galt
     */
    @Override
    public long processHandshake(Packet packet) {
        byte[] data;
        try {
            data = packet.getData();
        } catch (ClassCastException e) {
            return -1;
        }

        var reader = new ByteStreamReader(data);

        int expectedLen = reader.readInt16();

        try {
            return Long.parseLong(reader.readString(expectedLen));
        } catch (NumberFormatException e) {
            return -1;
        }
    }

    /**
     * Formaterer TeltonikaFMC data til et standard format
     *
     * @param packet TeltonikaFMC packet
     * @return Den formaterte dataen
     */
    @Override
    public Output processData(Packet packet) {
        byte[] data;
        try {
            data = packet.getData();
        } catch (ClassCastException e) {
            return null;
        }

        var reader = new ByteStreamReader(data);
        reader.skip(8);


        byte codec = reader.readInt8();

        if (codec != 0x08 && codec != (byte) 0x8e) {
            Logger.writeWarning("Feil codec: " + codec);
            return null;
        }

        var output = new Output(packet.getClient().getId());
        output.setDeviceFamily(packet.getDeviceFamily().toString());

        byte avlCount = reader.readInt8();

        for (var avlPacket = 0; avlPacket < avlCount; avlPacket++) {
            var packetContainer = new PacketContainer();

            long timestamp = reader.readInt64();
            packetContainer.setDateAndTime(timestamp);

            reader.skip(1);

            try {
                decodeGpsData(reader, packetContainer);

                if (codec == 0x08) reader.skip(1);
                else reader.skip(2);

                if (codec == 0x08) decodeIoDataCodec8(reader, packetContainer);
                else decodeIoDataCodec8e(reader, packetContainer);

            } catch (Exception e) {
                Logger.writeWarning("Noe galt med pakken -> " + e.getMessage());
                Logger.writeSubLine(Arrays.toString(reader.getStream()));
                return null;
            }

            output.addDataPacket(packetContainer);
        }

        return output;
    }

    /**
     * Leser GPS dataen fra enhetpakken og returnerer et gps element
     *
     * @param reader ByteStreamReader med byte-arrayen fra sensoren
     * @param packet Datapakken gps dataen skal legges til
     * @throws ArithmeticException nar det er noe galt med GPS dataen
     */
    private void decodeGpsData(ByteStreamReader reader, PacketContainer packet) throws ArithmeticException {
        var precision = 0.0000001;

        double longitude = reader.readInt32() * precision;

        double latitude = reader.readInt32() * precision;

        if (Math.abs(longitude) > 90 || Math.abs(latitude) > 90) {
            throw new ArithmeticException("Feil i GPS koordinatene: " + longitude + ", " + latitude);
        }

        short altitude = reader.readInt16();

        short angle = reader.readInt16();

        byte satellites = reader.readInt8();

        reader.skip(2);

        packet.setGpsElement(longitude, latitude, altitude, angle, satellites);
    }

    /**
     * Bruker codec8 for å decode IO elementene fra enhetpakken og returnere tolket data
     *
     * @param reader ByteStreamReader med byte-arrayen fra sensoren
     * @param packet Datapakken io dataen skal legges til
     * @throws ReflectiveOperationException videresender exception fra dekoderen
     */
    private void decodeIoDataCodec8(ByteStreamReader reader, PacketContainer packet) throws ReflectiveOperationException {
        var eventCount = reader.readUInt8();
        int t = reader.readInt8();
        for (var i = 0; i < eventCount + 3; i++) {
            if (i < t) {
                var eventId = reader.readUInt8();
                decoder.decode(eventId, reader, packet, parameters.getElement(eventId));
            } else t += reader.readUInt8() + 1;
        }
    }

    /**
     * Bruker codec8E for å decode IO elementene fra enhetpakken og returnere tolket data
     *
     * @param reader ByteStreamReader med byte-arrayen fra sensoren
     * @param packet Datapakken io dataen skal legges til
     * @throws ReflectiveOperationException videresender exception fra dekoderen
     */
    private void decodeIoDataCodec8e(ByteStreamReader reader, PacketContainer packet) throws ReflectiveOperationException {
        int eventCount = reader.readInt16();
        int t = reader.readInt16();
        for (var i = 0; i < eventCount + 4; i++) {
            if (i < t) {
                int eventId = reader.readInt16();
                decoder.decode(eventId, reader, packet, parameters.getElement(eventId));
            } else t += reader.readInt16() + 1;
        }
    }
}
