package main;

import database.Database;
import dataprocessing.output.Output;
import networking.Packet;
import dataprocessing.ProtocolHandler;
import networking.Server;
import tools.Cli;
import tools.Logger;


public class Main {
    private static Main instance;

    private boolean running = true;

    /**
     * Stiastisk funksjon som henter singleton intanse av Main.
     * Følger singleton lazy initialization.
     *
     * @return Main
     */
    public static Main getInstance() {
        if (instance == null) instance = new Main();
        return instance;
    }

    private Main() {
    }

    /**
     * Setter en boolean som stopper programmet
     */
    public void stop() {
        running = false;
    }

    /**
     * Initialiserer alle modulene og hjelpe klasser
     */
    private void init() {
        Logger.init();
        Server.getInstance().init();
        ProtocolHandler.getInstance().init();
        Database.getInstance().init("./db_credentials.properties");
        Cli.getInstance().init();
    }

    /**
     * Loopen til programmet
     */
    private void loop() {
        while (running) {
            var packet = Server.getInstance().getPacket();
            if (packet == null) continue;

            switch (ProtocolHandler.getInstance().validateData(packet)) {
                case DATA -> processPacket(packet);
                case HANDSHAKE -> processHandshake(packet);
                case ERROR -> packet.getClient().sendReceipt(null);
            }
        }
    }

    /**
     * Prosessering pipeline
     *
     * @param packet Pakken som skal prosesseres
     */
    private void processPacket(Packet packet) {
        Output data = ProtocolHandler.getInstance().processData(packet);
        data = Database.getInstance().processData(data);
        packet.getClient().sendReceipt(data);
    }

    /**
     * Handshake pipeline
     *
     * @param packet Handshake pakken som skal tolkes
     */
    private void processHandshake(Packet packet) {
        long id = ProtocolHandler.getInstance().processHandshake(packet);
        id = Database.getInstance().validateId(id);
        packet.getClient().acceptConnection(id);
    }

    /**
     * Avslutter programmet
     */
    private void close() {
        Server.getInstance().close();
    }


    public static void main(String[] args) {
        Main.getInstance().init();
        Main.getInstance().loop();
        Main.getInstance().close();
    }
}
