package networking;

import dataprocessing.output.Output;
import tools.Logger;

import java.io.*;
import java.net.Socket;
import java.net.SocketException;
import java.net.SocketTimeoutException;

/**
 * Abstarkt klient klasse som kjøres på egen tråd.
 * Sender og mottar data fra sensor.
 * Sørger for at komunikasjon med enhet følger forventet format
 */
public abstract class Client implements Runnable {
    protected final Server handler;

    protected final Socket socket;
    protected final DataInputStream input;
    protected final DataOutputStream output;

    private long id = -1;
    private int packetCount = 0;
    private final int bufferSize;
    private static final int INACTIVITY_TIMEOUT = 30;

    /**
     * Konstruktøren til klienten.
     *
     * @param handler   Serveren
     * @param socket    socketen til klienten
     * @param input     input strømen
     * @param output    output strømen
     * @param bufferSize maks størrelse på mottatte pakker
     */
    protected Client(Server handler, Socket socket, DataInputStream input, DataOutputStream output, int bufferSize) {
        this.handler = handler;
        this.socket = socket;
        this.input = input;
        this.output = output;
        this.bufferSize = bufferSize;
        try {
            socket.setSoTimeout(INACTIVITY_TIMEOUT * 60000);
        } catch (SocketException e) {
            e.printStackTrace();
        }
    }

    /**
     * Setter ID-en og godtar tilkoblingen.
     * Fra kobler klienten hvis ID-en er -1.
     * Kaller den abstarkte funksjone onAcceptConnection for å sende respons tilbake til enheten
     *
     * @param id ID-en
     */
    public void acceptConnection(long id) {
        if (socket.isClosed()) return;
        onAcceptConnection(id != -1);
        if (id == -1 || this.id != -1) {
            Logger.writeWarning(socket.getInetAddress().getHostAddress() + " ikke godkjent tilkobling");
            disconnect();
        }
        this.id = id;
    }

    /**
     * Sender responsen fra handshaken til sensoren.
     * Abstrakt funksjon som overskrives av arveklasser.
     *
     * @param flag Boolean som sier om klienten ble godkjent eller ikke
     */
    protected abstract void onAcceptConnection(boolean flag);


    /**
     * Hvis handshaken ikke har blitt gjennomført frakobles klienten
     * Kaller den abstarkte funksjone onSendReceipt for å sende respons tilbake til enheten
     *
     * @param data Dataen som har blitt tolket
     */
    public void sendReceipt(Output data) {
        if (socket.isClosed()) return;
        if (this.id != -1) onSendReceipt(data);
        else {
            onSendReceipt(null);
            Logger.writeWarning(socket.getInetAddress().getHostAddress() + " ikke godkjent tilkobling");
            disconnect();
        }
    }

    /**
     * Sender responsen til sensoren ved vanlige pakker.
     * Abstrakt funksjon som overskrives av arveklasser.
     *
     * @param data Dataen som har blitt tolket
     */
    protected abstract void onSendReceipt(Output data);


    /**
     * Mottar data fra sensoren som en bytestrøm.
     * Kaller den abstarkte funksjone onReceive for å omforme data til enhetens Packet type
     *
     * @return Packet med data
     * @throws IOException
     * @see Packet
     */
    protected Packet receive() throws IOException{
        var bytes = new byte[bufferSize];
        var i = input.read(bytes, 0, bufferSize);

        if (i < 0) throw new EOFException();

        var data = new byte[i];
        System.arraycopy(bytes, 0, data, 0, i);

        return onReceive(data);
    }

    /**
     * Abstarkt funksjon som omformer bytestrømen om til Enhetsfamilien sin Packet-objekt
     *
     * @param data byte-array
     * @return Enhetsfamilien sin Packet-objekt
     */
    protected abstract Packet onReceive(byte[] data);

    /**
     * Sender data til sensoren i form av en bytestrøm.
     *
     * @param data Byte-array med data
     */
    protected void send(byte[] data) {
        try {
            output.write(data);
            output.flush();
        } catch (IOException e) {
            Logger.writeWarning("Klarte ikke å sende data til: " + socket.getInetAddress().getHostAddress());
        }
    }

    /**
     * Koble fra sensor
     */
    public void disconnect() {
        try {
            output.close();
            socket.close();
            input.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Loopen til klienten
     */
    @Override
    public void run() {
        Logger.writeInfo("Åpnet tilkobling med: " + socket.getInetAddress().getHostAddress());
        handler.addClient(this);
        var stop = false;

        while (!stop) {
            Packet packet = null;

            try {
                packet = receive();
            } catch (SocketTimeoutException e) {
                Logger.writeInfo("Lukker tilkobling med " + socket.getInetAddress().getHostAddress() + " på grunn av inaktvitet");
                stop = true;
            } catch (IOException e) {
                stop = true;
            }

            packetCount++;
            handler.addPacket(packet);
        }

        disconnect();
        Logger.writeInfo("Lukket tilkobling med: " + socket.getInetAddress().getHostAddress());
        handler.removeClient(this);
    }

    /**
     * @return id til enheten
     */
    public long getId() {
        return id;
    }

    @Override
    public String toString() {
        return "ID: " + id + ", IP: " + socket.getInetAddress().getHostAddress() + ", Packets received: " + packetCount;
    }
}
