package networking;

import tools.Logger;

import java.io.*;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * Abstrakt Base-klasse for Lyttere. Lytter til porter og lager ny klient.
 */
public abstract class Listener implements Runnable {
    protected final Server handler;
    private ServerSocket serverSocket;
    private boolean listening;
    private final int port;


    /**
     * Konstruktør som setter port-nummeret til lytteren
     *
     * @param handler   Serveren
     * @param port      port-nummer til lytteren
     */
    protected Listener(Server handler, int port) {
        this.handler = handler;
        this.port = port;
        listening = false;
    }

    /**
     * Funksjon som initialisere lytteren
     */
    private void init() {
        startListening();
    }

    /**
     * Funksjon som starter starter lytteren.
     */
    private void startListening() {
        if (listening) {
            Logger.writeToConsole("Already listening");
            return;
        }

        Logger.writeToConsole("Trying to listen on port: " + port + "...");
        try {
            serverSocket = new ServerSocket(port);
            listening = true;
            Logger.writeToConsole("Listening on port: " + port);
        } catch (BindException e) {
            Logger.writeToConsole("Couldn't bind socket to port: " + port);
        } catch (IOException e) {
            Logger.writeToConsole("Something went wrong with starting listener");
        }
    }

    /**
     * Funksjon som stopper lytteren
     */
    private void stopListening() {
        if (!listening) {
            Logger.writeToConsole("Listener already stopped");
            return;
        }
        Logger.writeToConsole("Closing listener on port: " + port + "...");
        try {
            serverSocket.close();
            Logger.writeToConsole("Listener on port: " + port + " closed.");
        } catch (IOException e) {
            Logger.writeToConsole("Something went wrong when closing listener on port: " + port + ".");
            e.printStackTrace();
        }
        listening = false;
    }


    @Override
    public void run() {
        init();
        while (listening) connect();
        close();
    }

    /**
     * Sørger for å lage en ny klient tråd.
     * Blir overskrevet i underklasser av ServerListener
     *
     * @param socket Socket-objekt
     * @param input  Input-Objekt
     * @param output Output-Objekt
     */
    protected abstract Client createClient(Socket socket, DataInputStream input, DataOutputStream output);

    /**
     * Lytteren lytter etter enheter
     */
    private void connect() {
        try {
            var socket = serverSocket.accept();
            var input = new DataInputStream(socket.getInputStream());
            var output = new DataOutputStream(socket.getOutputStream());

            var client = createClient(socket, input, output);
            new Thread(client).start();

        } catch (IOException ignored) {
            //ignore
        }
    }

    /**
     * Slutt å lytte
     */
    public void close() {
        stopListening();
    }


    @Override
    public String toString() {
        if (listening)
            return "Listening on port: " + port;
        return "Closed";
    }
}