package networking;

import networking.teltonikafmc.TeltonikaFmcListener;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;


/**
 * Grensesnittet til Nettverksmodulen.
 * Starter lytterne
 */
public class Server {
    private static Server instance;

    private final List<Client> clients = Collections.synchronizedList(new ArrayList<>());
    private final List<Packet> packets = Collections.synchronizedList(new ArrayList<>());
    private final HashMap<String, Listener> listeners;

    /**
     * Stiastisk funksjon som henter singleton intanse av Server.
     * Følger singleton lazy initialization.
     *
     * @return Server
     */
    public static Server getInstance() {
        if (instance == null) instance = new Server();
        return instance;
    }

    /**
     * Konstruktøren til servern
     */
    private Server() {
        listeners = new HashMap<>();
    }

    /**
     * Legger til lytterne
     *
     * @see Listener
     */
    public void init() {
        addListener("TeltonikaFMC", new TeltonikaFmcListener(this));
    }

    /**
     * Stopper alle lytterene
     */
    public void close() {
        for (Listener listener : listeners.values()) listener.close();

        synchronized (clients) {
            for (Client client : clients) client.disconnect();
        }
    }

    /**
     * Legger til en pakke i en synkronisert liste
     * @param packet pakken
     */
    public void addPacket(Packet packet) {
        packets.add(packet);
    }

    /**
     * Henter pakken på toppen av listen
     *
     * @return Packet-objekt
     * @see Packet
     */
    public Packet getPacket() {
        if (packets.isEmpty()) return null;
        return packets.remove(0);
    }

    /**
     * Returnerer hvor mange pakker som venter på prosessering
     *
     * @return Antall pakker
     */
    public int getPacketQueueSize(){
        return packets.size();
    }

    /**
     * Legger til en ny lytter og starter den på en ny tråd
     *
     * @param listener lytteren
     */
    private void addListener(String name, Listener listener) {
        listeners.put(name.toLowerCase(), listener);
        new Thread(listener).start();
    }

    /**
     * Legger til en klient i en synkronisert liste
     *
     * @param client klienten
     */
    protected void addClient(Client client) {
        clients.add(client);
    }

    /**
     * Fjerner klienten fra listen
     * @param client klienten
     */
    protected void removeClient(Client client) {
        clients.remove(client);
    }

    /**
     * @return Antall tilkoblede klienter
     */
    public int getClientCount() {
        return clients.size();
    }

    /**
     * Henter klienten
     *
     * @param idx indeksen i listen
     * @return klienten
     */
    public Client getClient(int idx) {
        return clients.get(idx);
    }

    @Override
    public String toString() {
        var out = new StringBuilder("Connected clients: " + getClientCount());
        for (var entry : listeners.entrySet()) out.append("\n").append(entry.getKey()).append(":\t").append(entry.getValue());
        return out.toString();
    }
}
