package tools;

/**
 * Klasse for å lese data fra en byte strøm. Bruker funksjoner fra DataConverter
 * @see DataConverter
 */
public class ByteStreamReader {
    private final byte[] stream;
    private int readIdx = 0;


    /**
     * Konstruktøren til ByteStreamReader
     *
     * @param stream Byte-array
     */
    public ByteStreamReader(byte[] stream) {
        this.stream = stream;
    }

    /**
     * Leser en byte fra strømen
     *
     * @return byte
     */
    public byte readInt8() {
        return DataConverter.byteToInt8(stream, readIdx++);
    }

    /**
     * Leser en unsigned byte fra strømen
     *
     * @return int
     */
    public int readUInt8() {
        return readInt8() & 0xFF;
    }

    /**
     * Leser 2 byte fra strømen
     *
     * @return short
     */
    public short readInt16() {
        short value = DataConverter.byteToInt16(stream, readIdx);
        readIdx += 2;
        return value;
    }

    /**
     * Leser 4 byte fra strømen
     *
     * @return int
     */
    public int readInt32() {
        var value = DataConverter.byteToInt32(stream, readIdx);
        readIdx += 4;
        return value;
    }

    /**
     * Leser 8 byte fra strømen
     *
     * @return long
     */
    public long readInt64() {
        long value = DataConverter.byteToInt64(stream, readIdx);
        readIdx += 8;
        return value;
    }

    /**
     * Leser n antall byte fra strømen
     *
     * @param length antall byte
     * @return Byte-array
     */
    public byte[] readByteArray(int length) {
        var out = new byte[length];
        System.arraycopy(stream, readIdx, out, 0, length);
        return out;
    }

    /**
     * Leser en string fra strømen
     *
     * @param length lengden på stringen
     * @return String
     */
    public String readString(int length) {
        var out = new StringBuilder();
        for (var i = 0; i < length; i++)
            out.append((char) stream[readIdx++]);
        return out.toString();
    }

    /**
     * Hopper over n antall byte
     *
     * @param offset antall byte
     */
    public void skip(int offset) {
        readIdx += offset;
    }

    /**
     * Returnener strømen
     *
     * @return Byte-array
     */
    public byte[] getStream() {
        return stream;
    }
}
