package tools;

import main.Main;
import networking.Server;

import java.util.HashMap;
import java.util.Scanner;

/**
 * Singelton klasse med CLI kommadoer.
 * Kjører på egentråd og venter på kommadoer fra kommandolinjen.
 */
public class Cli extends Thread {
    /**
     * Interface for kommandoer
     */
    private interface Cmd {
        /**
         * Funksjon som kalles ved en kommando
         *
         * @param args liste med argumenter
         * @return tilbakemeldingen til bruker
         */
        String call(String... args);

        /**
         * @return Hjelp informasjonen
         */
        String help();
    }

    private static Cli instance;
    private final Scanner scanner;
    private final HashMap<String, Cmd> cmd;

    /**
     * Stiastisk funksjon som henter singleton intanse av Cli.
     * Følger singleton lazy initialization.
     *
     * @return Cli
     */
    public static Cli getInstance() {
        if (instance == null) instance = new Cli();
        return instance;
    }

    private Cli() {
        scanner = new Scanner(System.in);
        cmd = new HashMap<>();
        setDaemon(true);
    }

    /**
     * Initialiserer kommandoene
     */
    public void init() {
        add("Packetqueue", new Cmd() {
            @Override
            public String call(String... args) {
                return "Packages waiting for processing: " + Server.getInstance().getPacketQueueSize();
            }

            @Override
            public String help() {
                return "Prints out how many packages are waiting for processing";
            }
        });

        add("Disconnect", new Cmd() {
            @Override
            public String call(String... args) {
                var out = "";

                try {
                    var idx = Integer.parseInt(args[0]);
                    var client = Server.getInstance().getClient(idx);
                    Logger.writeToConsole("Are you sure? (y/n)");
                    String inn = scanner.nextLine();
                    if (inn.equalsIgnoreCase("y")) {
                        client.disconnect();
                        out += "Client kicked";
                    } else out += "Aborted";

                } catch (NumberFormatException | IndexOutOfBoundsException e) {
                    out += "Client not found";
                }

                return out;
            }

            @Override
            public String help() {
                return "Disconnect a client\n" +
                        "\t[Index] - Disconnect client with index, where index is a number.";
            }
        });

        add("Client", new Cmd() {
            @Override
            public String call(String... args) {
                var out = new StringBuilder();

                if (args.length == 0) {
                    out.append("Connected clients:");
                    for (var idx = 0; idx < Server.getInstance().getClientCount(); idx++)
                        out.append("\n\t[index: ").append(idx).append("] Client information: ").append(Server.getInstance().getClient(idx));
                    out.append("\nCount: ").append(Server.getInstance().getClientCount());
                } else {
                    try {
                        var idx = Integer.parseInt(args[0]);
                        out.append("[index: ").append(idx).append("] Client information: ").append(Server.getInstance().getClient(idx));
                    } catch (NumberFormatException | IndexOutOfBoundsException e) {
                        out.append("Client not found");
                    }
                }
                return out.toString();
            }

            @Override
            public String help() {
                return "Shows client information.\n" +
                        "\t[Index] - Show client information with index, where index is a number.";
            }
        });

        add("Network", new Cmd() {
            @Override
            public String call(String... args) {
                return Server.getInstance().toString();
            }

            @Override
            public String help() {
                return "Shows global network status.";
            }
        });

        add("exit", new Cli.Cmd() {
            @Override
            public String call(String... args) {
                Main.getInstance().stop();
                return "Closing...";
            }

            @Override
            public String help() {
                return "Close the program";
            }
        });

        add("Help", new Cmd() {
            @Override
            public String call(String... args) {
                var out = new StringBuilder();


                if (args.length == 0) {
                    out.append("Commands: \t");
                    for (String name : cmd.keySet()) out.append("\n\t").append(name.toUpperCase());
                } else {
                    if (cmd.containsKey(args[0])) out.append(args[0].toUpperCase()).append(": ").append(cmd.get(args[0]).help());
                    else out.append("Command not found");
                }
                return out.toString();
            }

            @Override
            public String help() {
                return "Show help information for commands\n" +
                        "\t[Command] - Show help information for given command";
            }
        });

        start();
    }

    /**
     * Legger til en ny kommando
     *
     * @param name Navnet på kommandoen
     * @param callback Cmd-objekt som inneholder funksjonen og hjelp information
     */
    public void add(String name, Cmd callback) {
        cmd.put(name.toLowerCase(), callback);
    }

    @Override
    public void run() {
        while (true) {
            String[] in = scanner.nextLine().toLowerCase().split("\\s+");
            String key = in[0];
            var args = new String[in.length - 1];
            System.arraycopy(in, 1, args, 0, args.length);

            if (cmd.containsKey(key)) Logger.writeToConsole(cmd.get(key).call(args));
            else Logger.writeToConsole("Command not found");
            Logger.writeToConsole("");
        }
    }
}
