package tools;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

/**
 * Klasse med statiske funksjoner for å skriver hendelse til en logg-fil.
 */
public class Logger {
    private Logger() {}

    private static final String FILE_NAME = "log.txt";
    private static boolean isInitialized = false;

    /**
     * Initierer loggeren, kalles i starten av programmet. Hvis filen finnes skjer det ingenting.
     * Hvis den ikke finnes, lages det en ny fil og skriver dato og tid i toppen av filen.
     */
    public static void init() {
        try {
            var logFile = new File(FILE_NAME);
            if (logFile.createNewFile()) writeInfo(FILE_NAME + " - Created: " + LocalDateTime.now());
            isInitialized = true;
        } catch (IOException e) {
            writeToConsole("Feil i Logger klasse");
            e.printStackTrace();
        }
    }

    /**
     * Skriv advarsler til loggen.
     *
     * @param logDetails Detaljene som skal skrives til loggen.
     */
    public static void writeWarning(String logDetails) {
        writeToLog(logDetails, "Warning");
    }

    /**
     * Skriv feilmeldinger til loggen.
     *
     * @param logDetails Detaljene som skal skrives til loggen.
     */
    public static void writeError(String logDetails) {
        writeToLog(logDetails, "ERROR");
    }

    /**
     * Skriv info til loggen.
     *
     * @param logDetails Detaljene som skal skrives til loggen.
     */
    public static void writeInfo(String logDetails) {
        writeToLog(logDetails, "Info");
    }


    /**
     * Skriv underlinje til loggen. (Ny linje med 4 mellomrom(tab)).
     *
     * @param logDetails Detaljene som skal skrives til loggen.
     */
    public static void writeSubLine(String logDetails) {
        if (!isInitialized) return;

        try (var writerObj = new FileWriter(FILE_NAME, true)) {
            writerObj.write("\t " + logDetails);
            writerObj.write(System.getProperty("line.separator"));

        } catch (IOException e) {
            writeToConsole("Klarte ikke skrive til " + FILE_NAME);
            e.printStackTrace();
        }
    }

    /**
     * Skriver hendelse til logg-filen. Skriver dato, tid, logg-info og type melding (Info, Advarsel, Feilmelding).
     * Kaller init() hvis dette ikke er gjort allerede.
     *
     * @param logDetails Detaljene som skal skrives til loggen.
     * @param type       Type melding (Info, Advarsel, Feilmelding).
     */
    private static void writeToLog(String logDetails, String type) {
        if (!isInitialized) init();

        try (var writerObj = new FileWriter(FILE_NAME, true)) {

            var date = LocalDateTime.now();
            var format = DateTimeFormatter.ofPattern("dd-MM-yyyy HH:mm:ss");
            String formattedDate = date.format(format);

            writerObj.write(formattedDate + " [" + type + "] " + logDetails);
            writerObj.write(System.getProperty("line.separator"));

        } catch (IOException e) {
            writeToConsole("Klarte ikke skrive til " + FILE_NAME);
            e.printStackTrace();
        }
    }

    public static void writeToConsole(String logDetails){
        System.out.println(logDetails);
    }
}
